package com.docus.sw.fenpan;

import com.alibaba.excel.util.FileUtils;
import com.docus.sw.Config;
import com.docus.sw.word.GetPicsDocx;
import com.docus.sw.word.MyFileUtil;
import com.docus.sw.word.PdfBoxUtils;
import com.docus.sw.word.ReadImgDoc;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.imaging.ImageInfo;
import org.apache.commons.imaging.ImageReadException;
import org.apache.commons.imaging.Imaging;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.*;
import java.util.concurrent.*;

@Slf4j
public class FenpanService {
    public void fenpan(String saveUrl, String readUrl) {
        try {
            FenpanService main = new FenpanService();
            Map<String, Zong> zongMap = main.readFile(readUrl);

            Map<Zong, List<Plate>> zongListMap = new HashMap<>();
            //读取需要转宗的文件夹
            Integer i = 0;
            for (String key : zongMap.keySet()) {
                List<Plate> plateList = new ArrayList<>();
                Zong zong = zongMap.get(key);
                List<Roll> rollList = zong.getRollList();
                Plate plate = new Plate(Integer.parseInt(Config.getParam("plate.size")), String.format("%04d", i++));
                for (Roll roll : rollList) {
                    Boolean put = plate.put(roll);
                    if (!put) {
                        //磁盘放不进去了
                        plateList.add(plate);
                        plate = new Plate(Integer.parseInt(Config.getParam("plate.size")), String.format("%04d", i++));
                        plate.put(roll);
                    }
                }
                plateList.add(plate);
                zongListMap.put(zong, plateList);
            }


            //copy 文件到新的目录
            for (Zong zong : zongListMap.keySet()) {

                File saveUrlFile = new File(saveUrl + zong.getName());
                if (!saveUrlFile.exists()) {
                    FileUtils.createDirectory(saveUrlFile);
                }

                for (Plate plate : zongListMap.get(zong)) {
                    for (Roll roll : plate.getRollList()) {
                        copyFolder(new File(roll.getAbsolutePath()), new File(saveUrlFile + "/" + plate.getName() + "/" + roll.getName()));
                    }
                }
            }

            //输出text
            for (Zong zong : zongListMap.keySet()) {
                System.out.println(zong.getName());
                for (Plate plate : zongListMap.get(zong)) {
                    System.out.print(plate.getName() + "   ---  ");
                    for (Roll roll : plate.getRollList()) {
                        System.out.print(roll.getName() + ",");
                    }
                    System.out.println();
                }
            }
        } catch (IOException e) {
            log.error("读取配置文件失败！", e);
            throw new RuntimeException(e);
        } catch (ExecutionException e) {
            throw new RuntimeException(e);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public static void main(String[] args) throws IOException {

        //读取配置文件。
        Config.loadConfig();
        String saveUrl = Config.getParam("file.save.url");
        String readUrl = Config.getParam("file.read.url");
        new FenpanService().fenpan(saveUrl, readUrl);
    }

    public List<Document> getDocument(Pieces piece) {
        return null;
    }


    public Map<String, Zong> readFile(String readUrl) throws ExecutionException, InterruptedException {
        //读取文件夹。
        List<Pieces> pieces = new ArrayList<>();
        Map<String, Roll> rollMap = new LinkedHashMap<>();
        Map<String, Zong> zongMap = new LinkedHashMap<>();
        this.findAllDir(readUrl, pieces);
        //写入文件

        List<Future<String>> futures2 = new ArrayList<>();
        ExecutorService executorService2 = Executors.newFixedThreadPool(2);
        for (Pieces piece : pieces) {

            Future<String> aa = executorService2.submit(() -> {

                //根据文件类型
                if (piece.getFileTypeEnum() == FileTypeEnum.DOC) {
                    //从word 直接提取图片
                    //提取图片为document ，然后用于后面判断
                    String teamName = "temp/" + UUID.randomUUID();
                    File file = new File(teamName);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    List<Document> documentList = new ArrayList<>();
                    ReadImgDoc.readPicture(piece.getAbsolutePath(), file.getAbsolutePath());
                    File[] files = file.listFiles();
                    for (File pdfImg : files) {
                        getDocumentList(documentList, pdfImg);
                    }

                    piece.put(documentList);
                    //删除对应的temp 文件
                    FileUtils.delete(file);
                } else if (piece.getFileTypeEnum() == FileTypeEnum.DOCX) {
                    //从word 直接提取图片
                    //提取图片为document ，然后用于后面判断
                    String teamName = "temp/" + UUID.randomUUID();
                    File file = new File(teamName);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    List<Document> documentList = new ArrayList<>();
                    GetPicsDocx.getPics(piece.getAbsolutePath(), file.getAbsolutePath());
                    File[] files = file.listFiles();
                    for (File pdfImg : files) {
                        getDocumentList(documentList, pdfImg);
                    }

                    piece.put(documentList);
                    //删除对应的temp 文件
                    FileUtils.delete(file);
                } else if (piece.getFileTypeEnum() == FileTypeEnum.PDF) {
                    // 从pdf 提取图片,
                    //提取图片为document ，然后用于后面判断
                    String teamName = "temp/" + UUID.randomUUID();
                    File file = new File(teamName);
                    if (!file.exists()) {
                        file.mkdirs();
                    }

                    List<Document> documentList = new ArrayList<>();
                    try {
                        PdfBoxUtils.pdf2image(piece.getAbsolutePath(), file.getAbsolutePath());
                        File[] files = file.listFiles();

                        for (File pdfImg : files) {
                            getDocumentList(documentList, pdfImg);
                        }
                    } catch (IOException e) {
                        log.error(e.getMessage(), e);
                    }

                    piece.put(documentList);
                    //删除对应的temp 文件
//                FileUtils.delete(file);
                } else {
                    //是图片，直接从图片提取
                    List<Document> documentList = new ArrayList<>();
                    File sourceFile = new File(piece.getAbsolutePath());
                    File[] files = sourceFile.listFiles();
                    ExecutorService executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());

                    List<Future<Document>> futures = new ArrayList<>();

//                    System.out.println("件名："+file.getName());
                    for (File pdfImg : files) {
//                        System.out.println("图片名"+pdfImg.getName());
                        Future<Document> future = executorService.submit(() -> getDocumentList(pdfImg));
                        futures.add(future);
                    }

                    for (Future<Document> future : futures) {
                        Document image = null;
                        try {
                            image = future.get();
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        } catch (ExecutionException e) {
                            throw new RuntimeException(e);
                        }
                        if (image != null) {
                            documentList.add(image);
                        }
                        // 处理图片
                    }

                    executorService.shutdown();
                }

                return "OK";
            });
            futures2.add(aa);

        }

        for (Future<String> f : futures2) {
            f.get();
        }

        executorService2.shutdown();

        for (Pieces piece : pieces) {
            //根据文件类型
            //填充卷
            File file = new File(piece.getAbsolutePath());
            File parentFile = file.getParentFile();
            String name = parentFile.getName();
            Roll roll = rollMap.get(name);
            if (roll == null) {
                roll = new Roll(name, parentFile.getAbsolutePath());
            }
            roll.putPieces(piece);
            rollMap.put(name, roll);


            String zongName = parentFile.getParentFile().getName();
            Zong zong = zongMap.get(zongName);
            if (zong == null) {
                zong = new Zong(parentFile.getParentFile().getAbsolutePath(), parentFile.getParentFile().getName());
            }
            zongMap.put(zongName, zong);
        }


        FileUtils.delete(new File("temp"));

        for (String zongName : zongMap.keySet()) {
            Zong zong = zongMap.get(zongName);
            File zongfile = new File(zong.getAbsolutePath());
            File[] files = zongfile.listFiles();
            // 创建自定义比较器
            Comparator<File> fileComparator = new Comparator<File>() {
                @Override
                public int compare(File file1, File file2) {
                    // 使用正则表达式提取数字部分
                    int num1 = extractNumber(file1.getName());
                    int num2 = extractNumber(file2.getName());

                    // 比较提取的数字部分
                    return Integer.compare(num1, num2);
                }

                private int extractNumber(String fileName) {
                    String numberPart = fileName.replaceAll("[^0-9]", "");
                    return numberPart.isEmpty() ? 0 : Integer.parseInt(numberPart);
                }
            };

            Arrays.sort(files, fileComparator);
            for (File file : files) {
                Roll roll = rollMap.get(file.getName());
                zong.put(roll);
            }
        }
        return zongMap;


    }

    private static void getDocumentList(List<Document> documentList, File file) {
        //非图片模式，跳过。
        if (!(file.getName().endsWith(".jpg") || file.getName().endsWith(".png") || file.getName().endsWith(".jpeg") || file.getName().endsWith(".tif") || file.getName().endsWith(".tiff")) || file.getName().endsWith(".jp2") || file.getName().endsWith(".jpm") || file.getName().endsWith(".gif")) {
            return;
        }

        if (file.getName().endsWith(".jp2") || file.getName().endsWith(".jpm")) {
            // 读取 JPEG 2000 图像文件

            try {
                BufferedImage image = ImageIO.read(file);
                int height = image.getHeight();
                int width = image.getWidth();
                Document document = new Document(width, height, 300,file.getName());
                documentList.add(document);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }

        } else {
            try {
                ImageInfo imageInfo = Imaging.getImageInfo(file);
                int height = imageInfo.getHeight();
                int width = imageInfo.getWidth();
                int physicalHeightDpi = imageInfo.getPhysicalHeightDpi();
                Document document = new Document(width, height, physicalHeightDpi,file.getName());
                documentList.add(document);
            } catch (IOException e) {
                FileUtils.delete(file);
                throw new RuntimeException("非图片格式", e);
            } catch (ImageReadException e) {
                FileUtils.delete(file);
                throw new RuntimeException(e);
            } catch (IllegalArgumentException e) {
                FileUtils.delete(file);
            }
        }
    }


    private static Document getDocumentList(File file) {
        //非图片模式，跳过。
        if (!(file.getName().endsWith(".jpg") || file.getName().endsWith(".png") || file.getName().endsWith(".jpeg") || file.getName().endsWith(".tif") || file.getName().endsWith(".tiff")) || file.getName().endsWith(".jp2") || file.getName().endsWith(".jpm") || file.getName().endsWith(".gif")) {
            return null;
        }

        if (file.getName().endsWith(".jp2") || file.getName().endsWith(".jpm")) {
            // 读取 JPEG 2000 图像文件

            try {
                BufferedImage image = ImageIO.read(file);
                int height = image.getHeight();
                int width = image.getWidth();
                Document document = new Document(width, height, 300,file.getName());
                return document;
            } catch (IOException e) {
                throw new RuntimeException(e);
            }

        } else {
            try {
                ImageInfo imageInfo = Imaging.getImageInfo(file);
                int height = imageInfo.getHeight();
                int width = imageInfo.getWidth();
                int physicalHeightDpi = imageInfo.getPhysicalHeightDpi();
                Document document = new Document(width, height, physicalHeightDpi,file.getName());
                return document;
            } catch (IOException e) {
                FileUtils.delete(file);
                throw new RuntimeException("非图片格式", e);
            } catch (ImageReadException e) {
                FileUtils.delete(file);
                throw new RuntimeException(e);
            } catch (IllegalArgumentException e) {
                FileUtils.delete(file);
            }
        }
        return null;
    }


    private void findAllDir(String absolutePath, List<Pieces> allDirectory) {
        File sourceFile = new File(absolutePath);
        File[] files = sourceFile.listFiles();
        for (File o : files) {
            if (o.isDirectory()) {
                findAllDir(o.getAbsolutePath(), allDirectory);
            }

            if (o.isFile()) {
                //判断是word,pdf,pic
                if (o.getName().endsWith(".pdf")) {
                    Pieces pieces = new Pieces(FileTypeEnum.PDF, o.getAbsolutePath(), o.getName());
                    allDirectory.add(pieces);
                } else if (o.getName().endsWith(".docx")) {
                    Pieces pieces = new Pieces(FileTypeEnum.DOCX, o.getAbsolutePath(), o.getName());
                    allDirectory.add(pieces);
                } else if (o.getName().endsWith(".doc")) {
                    Pieces pieces = new Pieces(FileTypeEnum.DOC, o.getAbsolutePath(), o.getName());
                    allDirectory.add(pieces);
                } else if (o.getName().endsWith(".jpg") || o.getName().endsWith(".png") || o.getName().endsWith(".jpeg") || o.getName().endsWith(".tif") || o.getName().endsWith(".tiff")) {
                    Pieces pieces = new Pieces(FileTypeEnum.JPG, o.getParentFile().getAbsolutePath(), o.getParentFile().getName());
                    allDirectory.add(pieces);
                    break;
                } else {

                    System.out.println("请删除无效的文件：" + o.getAbsolutePath());

                }

            }
        }
    }


    public static void copyFolder(File sourceFolder, File destinationFolder) throws IOException {
        if (!destinationFolder.exists()) {
            destinationFolder.mkdirs();
        }

        for (File file : sourceFolder.listFiles()) {
            if (file.isDirectory()) {
                copyFolder(file, newFile(destinationFolder, file));
            } else {
                Path sourcePath = file.toPath();
                Path destinationPath = newFile(destinationFolder, file).toPath();
                Files.copy(sourcePath, destinationPath, StandardCopyOption.REPLACE_EXISTING);
            }
        }
    }


    public static File newFile(File parentDir, File childFile) {
        return new File(parentDir, childFile.getName());
    }
}
