package com.docus.sw.word;

import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.filters.Canvas;
import net.coobird.thumbnailator.geometry.Positions;
import org.apache.pdfbox.contentstream.PDFStreamEngine;
import org.apache.pdfbox.contentstream.operator.Operator;
import org.apache.pdfbox.cos.COSBase;
import org.apache.pdfbox.cos.COSName;
import org.apache.pdfbox.io.MemoryUsageSetting;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDPageTree;
import org.apache.pdfbox.pdmodel.encryption.AccessPermission;
import org.apache.pdfbox.pdmodel.encryption.InvalidPasswordException;
import org.apache.pdfbox.pdmodel.encryption.PDEncryption;
import org.apache.pdfbox.pdmodel.encryption.SecurityHandler;
import org.apache.pdfbox.pdmodel.encryption.StandardProtectionPolicy;
import org.apache.pdfbox.pdmodel.encryption.StandardSecurityHandler;
import org.apache.pdfbox.pdmodel.graphics.PDXObject;
import org.apache.pdfbox.pdmodel.graphics.form.PDFormXObject;
import org.apache.pdfbox.pdmodel.graphics.image.PDImageXObject;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

/**
 * pdfbox 加解密
 */
public class PdfBoxUtils {

    private static final String vectorKey = "AesVKeyDocus@702";

    /**
     * 加密
     *
     * @param key
     * @param sourceFilePath
     * @param destFilePath
     * @throws Exception
     */
    public static void encrypt(String key, String sourceFilePath, String destFilePath) throws Exception {

        File file = new File(sourceFilePath);
        PDDocument load = PDDocument.load(file);

        AccessPermission permissions = new AccessPermission();
        // 是否可以插入/删除/旋转页面
        permissions.setCanAssembleDocument(false);
        // 是否可以复制和提取内容
        permissions.setCanExtractContent(false);

        permissions.setCanExtractForAccessibility(false);
        // 设置用户是否可以填写交互式表单字段（包括签名字段）
        permissions.setCanFillInForm(false);
        // 设置用户是否可以修改文档
        permissions.setCanModify(false);
        // 设置用户是否可以添加或修改文本注释并填写交互式表单字段，如果canModify()返回true，则创建或修改交互式表单字段（包括签名字段）。
        permissions.setCanModifyAnnotations(false);
        // 设置用户是否可以打印。
        permissions.setCanPrint(false);
        // 设置用户是否可以降级格式打印文档
        permissions.setCanPrintDegraded(false);

        StandardProtectionPolicy p = new StandardProtectionPolicy(key, key, permissions);
        SecurityHandler sh = new StandardSecurityHandler(p);
        sh.prepareDocumentForEncryption(load);
        PDEncryption encryptionOptions = new PDEncryption();
        encryptionOptions.setSecurityHandler(sh);

        load.setEncryptionDictionary(encryptionOptions);
        load.save(destFilePath);
        load.close();
    }

    /**
     * 解密
     *
     * @param key
     * @param sourceFilePath
     * @param destFilePath
     * @throws InvalidPasswordException
     * @throws IOException
     */
    public static void decrypt(String key, String sourceFilePath, String destFilePath) throws InvalidPasswordException, IOException {
        File file = new File(sourceFilePath);
        PDDocument load = PDDocument.load(file, key);
        load.setAllSecurityToBeRemoved(true);
        load.save(destFilePath);
        load.close();
    }


    /**
     * pdfbox pdf to image
     */
    public static class DefPdfToImageEngine extends PDFStreamEngine {
        public int imageNum = 1;

        private final String des;
        private final String name;

        public DefPdfToImageEngine(String name, String des) {
            this.des = des;
            this.name = name;
        }

        @Override
        protected void processOperator(Operator operator, List<COSBase> operands) throws IOException {
            String operatorName = operator.getName();
            if ("Do".equals(operatorName)) {
                COSName objName = (COSName) operands.get(0);
                PDXObject pdxObject = this.getResources().getXObject(objName);
                if (pdxObject instanceof PDImageXObject) {
                    PDImageXObject imageXObject = (PDImageXObject) pdxObject;
                    BufferedImage image = imageXObject.getImage();
                    BufferedOutputStream os = new BufferedOutputStream(new FileOutputStream(this.des +"/"+ this.name + "-" + this.imageNum + ".jpg"));
                    ImageIO.write(image,"jpg",os);
                    ++this.imageNum;
                } else if (pdxObject instanceof PDFormXObject) {
                    PDFormXObject form = (PDFormXObject) pdxObject;
                    this.showForm(form);
                }
            } else {
                super.processOperator(operator, operands);
            }

        }
    }

    /**
     * pdf to image
     *
     * @param src
     * @param des
     * @throws IOException
     */
    public static void pdf2image(String src, String des) throws IOException {
        File pdfFile = new File(src);
        PDDocument load = null;
        try{
            load = PDDocument.load(pdfFile, MemoryUsageSetting.setupMixed(128 * 1024));
            DefPdfToImageEngine imageEngine = new DefPdfToImageEngine(pdfFile.getName().substring(0, pdfFile.getName().lastIndexOf(".")), des);
            for (PDPage page : load.getPages()) {
                imageEngine.processPage(page);
            }
        }finally {
            if(load!=null){
                load.close();
            }
        }


    }


    /**
     * 指定提取哪一页
     *
     * @param src
     * @param des
     * @param pageNum
     * @throws IOException
     */
    public static void pdf2image(String src, String des, int pageNum) throws IOException {
        File pdfFile = new File(src);
        PDDocument load = PDDocument.load(pdfFile, MemoryUsageSetting.setupMixed(1024 * 1024));
        DefPdfToImageEngine imageEngine = new DefPdfToImageEngine(pdfFile.getName().substring(0, pdfFile.getName().lastIndexOf(".")), des);

        PDPageTree pages = load.getPages();

        for (int i = 0; i < pages.getCount(); i++) {

            if (pageNum == i + 1) {
                imageEngine.processPage(pages.get(i));
                break;
            }
        }

        load.close();
    }

    public static void main(String[] args) throws Exception {
        pdf2image("C:\\Users\\zhanghai\\Desktop\\桌面\\test\\a.pdf", "C:\\Users\\zhanghai\\Desktop\\桌面\\test\\");


//        long a1 = System.currentTimeMillis();
//        encrypt(vectorKey, "D:\\workspace\\docus\\业务梳理\\测试数据\\伟淞\\98m.pdf", "D:\\workspace\\docus\\业务梳理\\测试数据\\伟淞\\98m-encrypt.pdf");
//        long b1 = System.currentTimeMillis();
//        System.out.println("加密使用时间：" + (b1 - a1));
//
//        long a2 = System.currentTimeMillis();
//        decrypt(vectorKey, "D:\\workspace\\docus\\业务梳理\\测试数据\\伟淞\\98m-encrypt.pdf", "D:\\workspace\\docus\\业务梳理\\测试数据\\伟淞\\98m-decrypt.pdf");
//        long b2 = System.currentTimeMillis();
//        System.out.println("解密使用时间：" + (b2 - a2));
    }
}
